@@ -69,6 +69,12 @@ VARIABLES = [
     ["mm_mirror_rotation_y", "O.E. Rotation Y (Roll)", "deg"],
     ["mm_mirror_offset_z", "O.E. Offset Z", "l"],
     ["mm_mirror_rotation_z", "O.E. Rotation Z (Yaw)", "deg"],
+    ["sm_offset_x_mirr_ref_frame", "Offset X in O.E. reference frame", "l"],
+    ["sm_offset_y_mirr_ref_frame", "Offset Y in O.E. reference frame", "l"],
+    ["sm_offset_z_mirr_ref_frame", "Offset Z in O.E. reference frame", "l"],
+    ["sm_rotation_around_x", "rotation [CCW] around X", "deg"],
+    ["sm_rotation_around_y", "rotation [CCW] around Y", "deg"],
+    ["sm_rotation_around_z", "rotation [CCW] around Z", "deg"],
     ["slit_width_xaxis", "Slit width/x-axis", "l"],
     ["slit_height_zaxis", "Slit height/z-axis", "l"],
     ["thickness", "Absorber Thickness", "l"],
@@ -87,7 +93,7 @@ class ScanVariableLoopPoint(AbstractScanVariableLoopPoint):
     category = "User Defined"
     keywords = ["data", "file", "load", "read"]
 
-    variable_name_id = Setting(10)
+    variable_name_id = Setting(11)
 
     def __init__(self):
         super(ScanVariableLoopPoint, self).__init__()
